﻿IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ZZZ_Ahora_System_Health]') AND type in (N'P', N'PC'))
  DROP PROCEDURE [ZZZ_Ahora_System_Health]
GO
-- tabla temporal resultados
IF OBJECT_ID ('ZZZ_TabResult') IS NOT NULL DROP TABLE ZZZ_TabResult 
GO	
CREATE PROCEDURE [dbo].[ZZZ_Ahora_System_Health]
	@NombreBD nvarchar(100)=NULL,
	@TopN int=5,
	@SegMedicionContadores int=30,		-- maximo 1 minuto
	@JSON_Result NVARCHAR(MAX)=NULL OUTPUT
AS 
-- =============================================
-- #AUTOR:		Ceesi
-- #NAME:		ZZZ_Ahora_System_Health
-- #CREATION: 
--				20250304. #22319
-- #CLASIFICATION:
--				000-SISTEMA
-- #DESCRIPTION:
--				Comprueba configuracion de base de datos y servidor de SQL
-- #PARAMETERS: 
--				@NombreBD, se filtran los resultados por el nombre de la base de datos si no es NULL
-- #OBSERVATIONS:
--				1º devuelve caracteristicas del equipo 
--				2º devuelve caracteristicas de la instancia de SQL. Datos de licencia, version de SQL instalada, propiedades susceptibles de modificar: max server memory (MB), optimize for ad hoc workloads, cost threshold for parallelism ,max degree of parallelism
--				3º devuelve caracteristicas de la bases de dato susceptibles de ser modificadas. incluida Tempdb
--				4º comprobaciones de Indices
--				5º informacion de sesiones activas, suspendidas, transacciones
--				6º comprobaciones de cache
--				7º personalizaciones de codigo
--
--
-- #CHANGES: 
-- #EXAMPLE: 
-- =============================================
BEGIN TRY


Declare @collation_name_tempdb nvarchar(100)
Declare @show_advanced_options int
Declare @Nodos_Numa int, @Procesadores int, @MemoryFisica bigint, @socket_count int, @cores_per_socket int
Declare @committed_kb bigint,@committed_target_kb bigint
Declare @UltimoReinicio smalldatetime, @FechaMinPlanCache smalldatetime
Declare @NumInstancias int=0
Declare @NombreInstancia nvarchar(max)
Declare @lSql nvarchar(max)
Declare @BDActual nvarchar(150)
Declare @Script nvarchar(max)		
Declare @Detalles varchar(8000)	
Declare @PowerSchemeID NVARCHAR(255)
Declare @MachineName  NVARCHAR(500), @physical_memory_kb BIGINT, @sql_memory_model_desc NVARCHAR(500), @virtual_machine_type_desc  NVARCHAR(500)
Declare @cmd NVARCHAR(MAX)
Declare @filePath NVARCHAR(500)
Declare @CounterPrefix NVARCHAR(100) = CASE WHEN @@SERVICENAME = 'MSSQLSERVER' THEN 'SQLServer:' ELSE 'MSSQL$' + @@SERVICENAME + ':' END ; 
Declare @usuario varchar(150)=(SELECT USER)	
Declare @Cuenta AS INT

    SET NOCOUNT ON;
	SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
	
	-- comprobacion permisos usuario
	IF (SELECT IS_SRVROLEMEMBER('sysadmin', @usuario))=0 
			RAISERROR('El usuario debe tener permisos de administrador',12,1)
	
	
	SELECT @collation_name_tempdb = collation_name,@UltimoReinicio=create_date FROM sys.databases WHERE name='tempdb'
	SELECT @NombreInstancia=@@SERVERNAME
	-- comprobar que existe la bd
	SET @BDActual=@NombreBD
	IF @BDActual IS NOT NULL AND NOT EXISTS (SELECT 1 FROM sys.databases WHERE name=@BDActual) BEGIN
		RAISERROR('ERROR. La base de datos no existe',12,1)
	END
	IF @BDActual IS NULL
		SET @BDActual=DB_NAME()

	SELECT @SegMedicionContadores= CASE WHEN ISNULL(@SegMedicionContadores,30)<0 THEN 30 WHEN ISNULL(@SegMedicionContadores,30)>60 THEN 60 ELSE @SegMedicionContadores END 

	-- tabla temporal resultados
	IF OBJECT_ID ('ZZZ_TabResult') IS NOT NULL DROP TABLE ZZZ_TabResult 
	CREATE TABLE ZZZ_TabResult (Id int identity(1,1),Categoria varchar(150),SubCategoria varchar(150),Tipo varchar(300),Detalles nvarchar(4000),Valor nvarchar(max))

	IF OBJECT_ID('tempdb..#Instancias') IS NOT NULL DROP TABLE #Instancias;
	CREATE TABLE #Instancias (Instance_Number NVARCHAR(MAX),Instance_Name NVARCHAR(MAX),Data_Field NVARCHAR(MAX));

	IF OBJECT_ID('tempdb..#Estadisticas') IS NOT NULL DROP TABLE #Estadisticas;
	CREATE TABLE #Estadisticas (NombreEstadistica NVARCHAR(MAX),Tabla NVARCHAR(MAX), NumRegistros bigint, UltimaActualizacion datetime);

	IF OBJECT_ID('tempdb..#Indices') IS NOT NULL DROP TABLE #Indices;
	CREATE TABLE #Indices (PorcFragmentacion decimal(5,0), Indice NVARCHAR(MAX),Tabla NVARCHAR(MAX));


	-- comprobar caracteristicas del equipo --------------------------------------------------------------------------------------------------------------------------------------------
	------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
	SELECT @Nodos_numa=numa_node_count, @Procesadores=cpu_count, @MachineName=convert(varchar,SERVERPROPERTY('MachineName')), @physical_memory_kb =physical_memory_kb,
			@socket_count=socket_count, @cores_per_socket=cores_per_socket, @sql_memory_model_desc=sql_memory_model_desc,
			@virtual_machine_type_desc=virtual_machine_type_desc
			,@committed_kb = committed_kb
			,@committed_target_kb=committed_target_kb
	FROM sys.dm_os_sys_info

	SELECT @virtual_machine_type_desc= @virtual_machine_type_desc+' ' + CASE WHEN CHARINDEX(' on ',@@Version)>0 THEN  RIGHT (@@Version, LEN(@@Version)- CHARINDEX(' on ',@@Version)-3) ELSE '' END
	
	-- caracteristicas
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Equipo','Características','Nombre',@MachineName
	UNION
	SELECT 'Comprobaciones del Servidor','Equipo','Características','Sistema Operativo',@virtual_machine_type_desc
	UNION
	SELECT 'Comprobaciones del Servidor','Equipo','Características','Procesadores Lógicos',convert(varchar,@Procesadores)
	UNION
	SELECT 'Comprobaciones del Servidor','Equipo','Características','Memoria Física Total',convert(varchar,round(convert(decimal(38,2),@physical_memory_kb / 1024.0 / 1024.0),0)) +' GB' 
	UNION
	SELECT 'Comprobaciones del Servidor','Equipo','Características','Nº de Sockets' ,convert(varchar,@socket_count) 
	UNION
	SELECT 'Comprobaciones del Servidor','Equipo','Características','Nº de Cores por Socket', convert(varchar,@cores_per_socket) 
	UNION 
	SELECT 'Comprobaciones del Servidor','Equipo','Características','Nº de Nodos Numa', convert(varchar,@Nodos_numa)

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Equipo','Características','Memoria Fisica Disponible en el SOP', convert(varchar,round(convert(decimal(38,2),available_physical_memory_kb / 1024.0 / 1024.0),0)) +' GB' 
	FROM sys.dm_os_sys_memory 

	-- plan de energia   	
	EXEC xp_regread 'HKEY_LOCAL_MACHINE','SYSTEM\CurrentControlSet\Control\Power\User\PowerSchemes','ActivePowerScheme',@PowerSchemeID OUTPUT

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Equipo','Características','Plan de Energía activo', CASE @PowerSchemeID WHEN '8c5e7fda-e8bf-4a96-9a85-a6e23a8c635c' THEN 'Alto Rendimineto' 
																						WHEN 'a1841308-3541-4fab-bc81-f71556f20b4a' THEN 'Ahorro de energía'
																						WHEN '381b4222-f694-41f0-9685-ff5bb260df2e' THEN 'Equilibrado'
																						ELSE 'Plan Personalizado' END 


	-- uso CPU
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Equipo','Utilizacion CPU','Porcentaje uso CPU', CAST(100 - SystemIdle AS NVARCHAR(20)) + N'%.'
    FROM (
        SELECT record,
            record.value('(./Record/SchedulerMonitorEvent/SystemHealth/SystemIdle)[1]', 'int') AS SystemIdle
        FROM (
            SELECT TOP 1 CONVERT(XML, record) AS record
            FROM sys.dm_os_ring_buffers
            WHERE ring_buffer_type = N'RING_BUFFER_SCHEDULER_MONITOR'
            AND record LIKE '%<SystemHealth>%'
            ORDER BY timestamp DESC) AS rb
    ) AS y



	-- uso CPU
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Equipo','Utilizacion CPU','Porcentaje uso CPU por procesos no SQL',  CONVERT(NVARCHAR(100),100 - (y.SQLUsage + y.SystemIdle)) + N'%'
	FROM (
        SELECT record,
            record.value('(./Record/SchedulerMonitorEvent/SystemHealth/SystemIdle)[1]', 'int') AS SystemIdle
			,record.value('(./Record/SchedulerMonitorEvent/SystemHealth/ProcessUtilization)[1]', 'int') AS SQLUsage
        FROM (
            SELECT TOP 1 CONVERT(XML, record) AS record
            FROM sys.dm_os_ring_buffers
            WHERE ring_buffer_type = N'RING_BUFFER_SCHEDULER_MONITOR'
            AND record LIKE '%<SystemHealth>%'
            ORDER BY timestamp DESC) AS rb
    ) AS y

	-- espacio en los discos
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT DISTINCT  'Comprobaciones del Servidor','Equipo','Espacio disponible en disco','Unidad '+ convert(varchar,vs.volume_mount_point), convert(varchar,convert(decimal(38,0),vs.available_bytes/1024.0/1024.0/1024.0)) + ' GB'
	FROM sys.master_files AS f CROSS APPLY sys.dm_os_volume_stats(f.database_id, f.file_id) AS vs


	-- comprobar instancia servidor SQL ---------------------------------------------------------------------------------------------------------------------------------------------------
	---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
	INSERT INTO #Instancias (Instance_Number, Instance_Name, Data_Field)
	EXEC master.sys.xp_regread @rootkey = 'HKEY_LOCAL_MACHINE',@key = 'SOFTWARE\Microsoft\Microsoft SQL Server',@value_name = 'InstalledInstances'
	SELECT @NumInstancias=COUNT(*) FROM #Instancias
	-- instancias
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Datos generales Instancia', 'Nombre' , @NombreInstancia

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Datos generales Instancia', 'Nº de instancias ',convert(varchar,@NumInstancias)

	-- nº de bases de datos y tamaño total
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Datos generales Instancia','Nº de bases de datos de Usuario', convert(varchar,count(distinct DB_NAME(database_id))) + ' con un tamaño aproximado de ' + convert(varchar,convert(decimal(38,2),(SUM(size)* 8.0 / 1024.0 / 1024.0))) + ' GB' 
	FROM sys.master_files WHERE /*type_desc = 'ROWS' AND*/ DB_NAME(database_id) not in ('master','model','msdb','tempdb')

	-- nivel de compatibilidad de la master
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Datos generales Instancia', 'Nivel Compatibilidad base de datos Master' , convert(varchar,Compatibility_level) 
	FROM sys.databases WHERE name='master'

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Datos generales Instancia','Memoria confirmada por el administrador SQL (committed_kb)', convert(varchar,round(convert(decimal(38,2),@committed_kb / 1024.0 / 1024.0),0)) +' GB' 

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Datos generales Instancia','Memoria necesaria estimada por el administrador SQL (committed_target_kb)', convert(varchar,round(convert(decimal(38,2),@committed_target_kb / 1024.0 / 1024.0),0)) +' GB' 

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Datos generales Instancia','Modelo de Memoria SQL', convert(varchar,@sql_memory_model_desc) 

	   	
	SELECT @show_advanced_options=convert(bit,Value) FROM sys.configurations WHERE name='show advanced options'
	IF OBJECT_ID ('Tempdb..#TabAux') IS NOT NULL DROP TABLE #TabAux 
	CREATE TABLE #TabAux (LogDate datetime,ProcessInfo nvarchar(max),Text nvarchar(max))
	INSERT INTO #TabAux (LogDate,ProcessInfo,Text) 							
	EXEC sp_readerrorlog 0,1,'licensing'	
	
	-- licensing
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo, Detalles, Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Datos generales Instancia', 'Licencia SQL', Text 
	FROM #TabAux

	-- version SQL
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server', 'Datos generales Instancia','Versión SQL', CASE WHEN SERVERPROPERTY('EngineEdition') IN (5,8) THEN 'AZURE. ' ELSE '' END + ISNULL(convert(varchar, SERVERPROPERTY('Edition')),'') +' '+ ISNULL(convert(varchar,SERVERPROPERTY('ProductLevel')),'') +' '+ ISNULL(convert(varchar,SERVERPROPERTY('ProductUpdateLevel')),'') +' '+ISNULL(convert(varchar,SERVERPROPERTY('ProductVersion')),'') 
				
	-- Último Reinicio
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server', 'Datos generales Instancia','Último Reinicio Instancia SQL',convert(varchar,@UltimoReinicio,113) 
	
	-- max server memory (MB)
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Propiedades','Memoria Máxima Configurada',convert(varchar,convert(decimal(38,2),convert(numeric,value)/1024.0)) + ' GB'
	FROM sys.configurations WHERE name = 'max server memory (MB)'


	-- optimize for ad hoc workloads
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Propiedades', 'Optimizar Cargas de Trabajo Ad-Hoc',convert(varchar,Value)
	FROM sys.configurations WHERE name='optimize for ad hoc workloads' 

	-- cost threshold for parallelism
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Propiedades', 'Umbral de Costo para Paralelismo',convert(varchar,Value)
	FROM sys.configurations WHERE name='cost threshold for parallelism'
	
	--max degree of parallelism
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Propiedades', 'Grado de Paralelismo Máximo',convert(varchar,Value)
	FROM sys.configurations WHERE name='max degree of parallelism' 


	-- contadores de Rendimiento ---------------------------------------------------------------------------------------------------------------------
	-- Page life expectancy. Promedio de segundos que una pagina permanece en memoria (sobre 300). Ultimo valor observado
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Page life expectancy (segundos que una página permanece en memoria) ',convert(varchar,cntr_value)
	FROM sys.dm_os_performance_counters 
	WHERE counter_name = 'Page life expectancy' AND object_name = @CounterPrefix +'Buffer Manager'

	-- Average Wait Time (ms)
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
  	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Average Wait Time (ms) (promedio de espera por bloqueos)',convert(varchar,((dmv1.cntr_value/dmv2.cntr_value)))
    FROM sys.dm_os_performance_counters dmv1 
                    INNER JOIN sys.dm_os_performance_counters dmv2 ON dmv1.[object_name] = dmv2.[object_name] 
	WHERE dmv1.counter_name = 'Average Wait Time (ms)' AND dmv1.object_name = @CounterPrefix +'Locks' AND dmv1.instance_name ='_Total' 
                    AND dmv2.counter_name =  'Average Wait Time Base' AND dmv2.object_name = @CounterPrefix +'Locks' AND dmv2.instance_name ='_Total'

	-- Buffer cache hit ratio
	-- la caché satisface las solicitudes de datos sin necesidad de acceder al disco. Es un porcentaje. > 99%
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Buffer cache hit ratio (porcentaje de aciertos de páginas de datos)',convert(varchar,((dmv1.cntr_value*100/dmv2.cntr_value*100)/100))
    FROM sys.dm_os_performance_counters dmv1 
                    INNER JOIN sys.dm_os_performance_counters dmv2 ON dmv1.[object_name] = dmv2.[object_name]
   WHERE dmv1.counter_name = 'Buffer cache hit ratio' AND dmv1.object_name = @CounterPrefix +'Buffer Manager'
                    AND dmv2.counter_name =  'Buffer cache hit ratio base' AND dmv1.object_name = @CounterPrefix +'Buffer Manager'

	-- cache hit ratio
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Cache Hit Ratio (porcentaje de aciertos' + CASE WHEN RTRIM(dmv1.[instance_name])<>'_Total' THEN ' de ' ELSE ' ' END + RTRIM(dmv1.[instance_name]) +')' ,convert(varchar,((dmv1.cntr_value*100/dmv2.cntr_value*100)/100))
	FROM sys.dm_os_performance_counters dmv1 
				INNER JOIN sys.dm_os_performance_counters dmv2 ON dmv1.[object_name] = dmv2.[object_name] AND dmv1.[instance_name] = dmv2.[instance_name]
    WHERE dmv1.counter_name = 'Cache Hit Ratio'
                    AND dmv2.counter_name =  'Cache Hit Ratio Base'
                    AND dmv2.cntr_value <> 0
                    AND dmv1.[object_name] = @CounterPrefix + 'Plan Cache' 
                    AND dmv2.[object_name] = @CounterPrefix +'Plan Cache'

	-- Lazy writes/sec
	-- ACUMULATIVO cantidad de veces por segundo que el motor de base de datos mueve páginas modificadas (dirty pages) desde la memoria caché al disco para libera espacio en la memoria cuando es necesario.
	Declare @Cont_Lazywritessec_INI bigint,@Cont_Lazywritessec_FIN bigint
	SELECT @Cont_Lazywritessec_INI=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE counter_name = 'Lazy writes/sec' AND object_name = @CounterPrefix +'Buffer Manager'   

	-- Log Flush Wait Time
	-- ACUMULATIVO, tiempo que tarda en escribir un registro del log, despues de que se haya confirmado, a disco en milisegundos
	Declare @Cont_LogFlushWaitTime_INI bigint,@Cont_LogFlushWaitTime_FIN bigint
	SELECT @Cont_LogFlushWaitTime_INI=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE counter_name = 'Log Flush Wait Time' AND object_name = @CounterPrefix +'Databases' AND instance_name=@BDActual  

	Declare @Cont_LogFlushWaitTime_INI_TempDB int,@Cont_LogFlushWaitTime_FIN_TempDB int
	SELECT @Cont_LogFlushWaitTime_INI_TempDB=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE counter_name = 'Log Flush Wait Time' AND object_name = @CounterPrefix +'Databases' AND instance_name='tempDB'

	-- Checkpoint pages/sec
	-- páginas sucias que se escriben en el disco durante un checkpoint
	Declare @Cont_Checkpointpagessec_INI bigint,@Cont_Checkpointpagessec_FIN bigint
	SELECT @Cont_Checkpointpagessec_INI=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE counter_name = 'Checkpoint pages/sec' AND object_name = @CounterPrefix +'Buffer Manager'   

	-- Batch Requests/sec  
	-- cantidad de lotes de consultas procesadas por segundo
	Declare @Cont_BatchRequestssec_INI bigint,@Cont_BatchRequestssec_FIN bigint
	SELECT @Cont_BatchRequestssec_INI=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Batch Requests/sec' AND object_name = @CounterPrefix +'SQL Statistics'  

	-- SQL Compilations/sec
	-- nº de compilaciones por segundo (incluye las recompilaciones). Mucho consumo de CPU estadisticas desactualiadas? frente a una buena reutilizacion de los planes
	-- debería ser menor al 10% del número de Batch Requests/sec.
	Declare @Cont_SQLCompilationssec_INI bigint,@Cont_SQLCompilationssec_FIN bigint
	SELECT @Cont_SQLCompilationssec_INI=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'SQL Compilations/sec' AND object_name = @CounterPrefix +'SQL Statistics'  

	-- SQL Re-Compilations/sec                                                                                                         
	-- nº de recompilaciones.  Presión en la caché de planes, lo que obliga a SQL Server a eliminar planes antiguos?
	Declare @Cont_SQLReCompilationssec_INI bigint,@Cont_SQLReCompilationssec_FIN bigint
	SELECT @Cont_SQLReCompilationssec_INI=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'SQL Re-Compilations/sec' AND object_name = @CounterPrefix +'SQL Statistics'  

	-- Transactions/sec
	Declare @Cont_Transactionssec_INI int,@Cont_transactionssec_FIN int
	SELECT @Cont_Transactionssec_INI=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Transactions/sec' AND object_name = @CounterPrefix +'Databases'  AND instance_name=@BDActual  

	Declare @Cont_Transactionssec_INI_TempDB bigint,@Cont_Transactionssec_FIN_TempDB bigint
	SELECT @Cont_Transactionssec_INI_TempDB=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Transactions/sec' AND object_name = @CounterPrefix +'Databases'  AND instance_name='TempDB'  

	-- Log Flushes/sec 
	-- mide la cantidad de veces por segundo que el log de transacciones se descarga a disco, cada vez que una transacción se confirma.
	Declare @Cont_LogFlushessec_INI bigint,@Cont_LogFlushessec_FIN bigint
	SELECT @Cont_LogFlushessec_INI=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Log Flushes/sec' AND object_name = @CounterPrefix +'Databases'  AND instance_name=@BDActual  

	Declare @Cont_LogFlushessec_INI_TempDB int,@Cont_LogFlushessec_FIN_TempDB int
	SELECT @Cont_LogFlushessec_INI_TempDB=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Log Flushes/sec' AND object_name = @CounterPrefix +'Databases'  AND instance_name='TempDB'  

	--Lock Waits/sec. indica cuántas veces por segundo una operación en la base de datos se ve obligada a esperar debido a bloqueos en los recursos.
	Declare @Cont_LockWaitssec_INI bigint,@Cont_LockWaitssec_Fin bigint
	SELECT @Cont_LockWaitssec_INI=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Lock Waits/sec' AND object_name = @CounterPrefix +'Locks'  AND instance_name='_Total'

	-- Lock Wait Time (ms). mide el tiempo total, en milisegundos, que las solicitudes de bloqueo han pasado esperando a que el recurso esté disponible (tiempo han estado las transacciones en espera debido a bloqueos en la base de datos)
	Declare @Cont_LockWaitTime_INI bigint,@Cont_LockWaitTime_FIN bigint
	SELECT @Cont_LockWaitTime_INI=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Lock Wait Time (ms)' AND object_name = @CounterPrefix +'Locks'  AND instance_name='_Total'

	--Number of Deadlocks/sec. mide la cantidad de interbloqueos que ocurren por segundo                                                                                                         
	Declare @Cont_Deadlocks_INI int,@Cont_Deadlocks_FIN int
	SELECT @Cont_Deadlocks_INI=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Number of Deadlocks/sec' AND object_name = @CounterPrefix +'Locks'  AND instance_name='_Total'


	-- Page Reads/sec. Número de páginas leídas desde el disco por segundo.
	Declare @Cont_Pagereadssec_INI bigint,@Cont_Pagereadssec_FIN bigint
	SELECT @Cont_Pagereadssec_INI=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Page reads/sec' AND object_name = @CounterPrefix +'Buffer Manager'  

	-- Page Writes/sec. Número de páginas escritas en el disco por segundo.
	Declare @Cont_PageWritessec_INI bigint,@Cont_PageWritessec_FIN bigint
	SELECT @Cont_PageWritessec_INI=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Page Writes/sec' AND object_name = @CounterPrefix +'Buffer Manager' 

	-- Full Scans/sec. nº de scans completos de tablas o indices    
	Declare @Cont_FullScanssec_INI bigint,@Cont_FullScanssec_FIN bigint
	SELECT @Cont_FullScanssec_INI=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Full Scans/sec' AND object_name = @CounterPrefix +'Access Methods' 

	-- Index Searches/sec. nº de busquedas de indices por segundo. veces que se estan utilizando los indices
	Declare @Cont_IndexSearchessecc_INI bigint,@Cont_IndexSearchessecc_FIN bigint
	SELECT @Cont_IndexSearchessecc_INI=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Index Searches/sec' AND object_name = @CounterPrefix +'Access Methods' 
		
	-- Page Splits/sec
	Declare @Cont_PageSplitssec_INI bigint,@Cont_PageSplitssec_FIN bigint
	SELECT @Cont_PageSplitssec_INI=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Page Splits/sec' AND object_name = @CounterPrefix +'Access Methods' 

	-- Temp Tables Creation Rate
	Declare @Cont_TempTablesCreationRate_INI bigint,@Cont_TempTablesCreationRate_Fin bigint
	SELECT @Cont_TempTablesCreationRate_INI=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Temp Tables Creation Rate' AND object_name = @CounterPrefix +'General Statistics'


	----------------
	EXEC('WAITFOR DELAY ''00:00:30''')

	SELECT @Cont_Lazywritessec_FIN=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE counter_name = 'Lazy writes/sec' AND object_name = @CounterPrefix +'Buffer Manager'   
	SELECT @Cont_Lazywritessec_FIN=@Cont_Lazywritessec_FIN-@Cont_Lazywritessec_INI 

	SELECT @Cont_LogFlushWaitTime_FIN=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE counter_name = 'Log Flush Wait Time' AND object_name = @CounterPrefix +'Databases' AND instance_name=@BDActual  
	SELECT @Cont_LogFlushWaitTime_FIN=@Cont_LogFlushWaitTime_FIN-@Cont_LogFlushWaitTime_INI 

	SELECT @Cont_LogFlushWaitTime_FIN_TempDB=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE counter_name = 'Log Flush Wait Time' AND object_name = @CounterPrefix +'Databases' AND instance_name='tempDB'
	SELECT @Cont_LogFlushWaitTime_FIN_TempDB=@Cont_LogFlushWaitTime_FIN_TempDB-@Cont_LogFlushWaitTime_INI_TempDB 

	SELECT @Cont_Checkpointpagessec_FIN=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE counter_name = 'Checkpoint pages/sec' AND object_name = @CounterPrefix +'Buffer Manager' 
	SELECT @Cont_Checkpointpagessec_FIN=@Cont_Checkpointpagessec_FIN-@Cont_Checkpointpagessec_INI 	

	SELECT @Cont_BatchRequestssec_FIN=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Batch Requests/sec' AND object_name = @CounterPrefix +'SQL Statistics'  
	SELECT @Cont_BatchRequestssec_FIN=@Cont_BatchRequestssec_FIN-@Cont_BatchRequestssec_INI		

	SELECT @Cont_SQLCompilationssec_FIN=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'SQL Compilations/sec' AND object_name = @CounterPrefix +'SQL Statistics'  
	SELECT @Cont_SQLCompilationssec_FIN=@Cont_SQLCompilationssec_FIN-@Cont_SQLCompilationssec_INI		

	SELECT @Cont_SQLReCompilationssec_FIN=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'SQL Re-Compilations/sec' AND object_name = @CounterPrefix +'SQL Statistics'  
	SELECT @Cont_SQLReCompilationssec_FIN=@Cont_SQLReCompilationssec_FIN-@Cont_SQLReCompilationssec_INI	

	SELECT @Cont_transactionssec_FIN=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Transactions/sec' AND object_name = @CounterPrefix +'Databases'  AND instance_name=@BDActual  
	SELECT @Cont_transactionssec_FIN=@Cont_transactionssec_FIN-@Cont_Transactionssec_INI	

	SELECT @Cont_Transactionssec_FIN_TempDB=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Transactions/sec' AND object_name = @CounterPrefix +'Databases'  AND instance_name='TempDB'  
	SELECT @Cont_Transactionssec_FIN_TempDB=@Cont_Transactionssec_FIN_TempDB-@Cont_Transactionssec_INI_TempDB	

	SELECT @Cont_LogFlushessec_FIN=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Log Flushes/sec' AND object_name = @CounterPrefix +'Databases'  AND instance_name=@BDActual  
	SELECT @Cont_LogFlushessec_FIN=@Cont_LogFlushessec_FIN-@Cont_LogFlushessec_INI	

	SELECT @Cont_LogFlushessec_FIN_TempDB=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Log Flushes/sec' AND object_name = @CounterPrefix +'Databases'  AND instance_name='TempDB'  
	SELECT @Cont_LogFlushessec_FIN_TempDB=@Cont_LogFlushessec_FIN_TempDB-@Cont_LogFlushessec_FIN_TempDB	

	--Lock Waits/sec. indica cuántas veces por segundo una operación en la base de datos se ve obligada a esperar debido a bloqueos en los recursos.
	SELECT @Cont_LockWaitssec_Fin=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Lock Waits/sec' AND object_name = @CounterPrefix +'Locks'  AND instance_name='_Total'
	SELECT @Cont_LockWaitssec_Fin=@Cont_LockWaitssec_Fin-@Cont_LockWaitssec_INI	

	-- Lock Wait Time (ms). mide el tiempo total, en milisegundos, que las solicitudes de bloqueo han pasado esperando a que el recurso esté disponible (tiempo han estado las transacciones en espera debido a bloqueos en la base de datos)
	SELECT @Cont_LockWaitTime_FIN=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Lock Wait Time (ms)' AND object_name = @CounterPrefix +'Locks'  AND instance_name='_Total'
	SELECT @Cont_LockWaitTime_FIN=@Cont_LockWaitTime_FIN-@Cont_LockWaitTime_INI	

	--Number of Deadlocks/sec. mide la cantidad de interbloqueos que ocurren por segundo                                                                                                         
	SELECT @Cont_Deadlocks_FIN=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Number of Deadlocks/sec' AND object_name = @CounterPrefix +'Locks'  AND instance_name='_Total'
	SELECT @Cont_Deadlocks_FIN=@Cont_Deadlocks_FIN-@Cont_Deadlocks_INI		   	  

	-- Page Reads/sec. Número de páginas leídas desde el disco por segundo.
	SELECT @Cont_Pagereadssec_FIN=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Page reads/sec' AND object_name = @CounterPrefix +'Buffer Manager'  
	SELECT @Cont_Pagereadssec_FIN=@Cont_Pagereadssec_FIN-@Cont_Pagereadssec_INI	

	-- Page Writes/sec. Número de páginas escritas en el disco por segundo.
	SELECT @Cont_PageWritessec_FIN=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Page Writes/sec' AND object_name = @CounterPrefix +'Buffer Manager' 
	SELECT @Cont_PageWritessec_FIN=@Cont_PageWritessec_FIN-@Cont_PageWritessec_INI	


	SELECT @Cont_FullScanssec_FIN=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Full Scans/sec' AND object_name = @CounterPrefix +'Access Methods' 
	SELECT @Cont_FullScanssec_FIN=@Cont_FullScanssec_FIN-@Cont_FullScanssec_INI		

	-- Index Searches/sec. nº de busquedas de indices por segundo. veces que se estan utilizando los indices
	SELECT @Cont_IndexSearchessecc_FIN=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Index Searches/sec' AND object_name = @CounterPrefix +'Access Methods' 
	SELECT @Cont_IndexSearchessecc_FIN=@Cont_IndexSearchessecc_FIN-@Cont_IndexSearchessecc_INI

	-- Page Splits/sec
	SELECT @Cont_PageSplitssec_FIN=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Page Splits/sec' AND object_name = @CounterPrefix +'Access Methods' 
	SELECT @Cont_PageSplitssec_FIN=@Cont_PageSplitssec_FIN-@Cont_PageSplitssec_INI

	-- Temp Tables Creation Rate
	SELECT @Cont_TempTablesCreationRate_Fin=cntr_value
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name = 'Temp Tables Creation Rate' AND object_name = @CounterPrefix +'General Statistics'
	SELECT @Cont_TempTablesCreationRate_Fin=@Cont_TempTablesCreationRate_Fin-@Cont_TempTablesCreationRate_INI


	----
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Lazy writes/sec últimos ' + convert(varchar,@SegMedicionContadores)+ ' segs (veces por segundo que se escriben páginas sucias a disco para liberar memoria) ',convert(varchar,@Cont_Lazywritessec_FIN)

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Checkpoint pages/sec últimos ' + convert(varchar,@SegMedicionContadores) + ' segs (nº de paginas sucias que se escriben a disco en un checkpoint) ',convert(varchar,@Cont_Checkpointpagessec_FIN)
	
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Batch Requests/sec últimos ' + convert(varchar,@SegMedicionContadores)	+ ' segs (cantidad de lotes de consultas o batches procesadas por segundo) ',convert(varchar,@Cont_BatchRequestssec_FIN)

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'SQL Compilations/sec últimos ' + convert(varchar,@SegMedicionContadores)+ ' segs (nº de compilaciones por segundo) ',convert(varchar,@Cont_SQLCompilationssec_FIN)
	
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'SQL Re-Compilations/sec últimos ' + convert(varchar,@SegMedicionContadores)+ ' segs (nº de recompilaciones por segundo) ',convert(varchar,@Cont_SQLReCompilationssec_FIN)

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Lock Waits/sec últimos ' + convert(varchar,@SegMedicionContadores)+ ' segs (nº de solicitudes en espera debvido a bloqueos por segundo) ',convert(varchar,@Cont_LockWaitssec_Fin)

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Lock Wait Time (ms) últimos ' + convert(varchar,@SegMedicionContadores)+ ' segs (tiempo (ms) que las solicitudes has esperado debido a bloqueos) ',convert(varchar,@Cont_LockWaitTime_FIN)

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Number of Deadlocks/sec últimos ' + convert(varchar,@SegMedicionContadores)+ ' segs (nº de interbloqueos por segundo) ',convert(varchar,@Cont_Deadlocks_FIN)

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Page reads/sec últimos ' + convert(varchar,@SegMedicionContadores)+ ' segs (nº de páginas leídas desde el disco por segundo) ',convert(varchar,@Cont_Pagereadssec_FIN)

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Page Writes/sec últimos ' + convert(varchar,@SegMedicionContadores)+ ' segs (nº de páginas escritas en el disco por segundo) ',convert(varchar,@Cont_PageWritessec_FIN)

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Full Scans/sec últimos ' + convert(varchar,@SegMedicionContadores)+ ' segs (nº de exploraciones completas de tablas o índices que ocurren por segundo) ',convert(varchar,@Cont_FullScanssec_FIN)

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Index Searches/sec últimos ' + convert(varchar,@SegMedicionContadores)+ ' segs (nº de busquedas o utilizaciones de indices por segundo) ',convert(varchar,@Cont_IndexSearchessecc_FIN)

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Page Splits/sec últimos ' + convert(varchar,@SegMedicionContadores)+ ' segs (nº de divisiones de pagina por segundo) ',convert(varchar,@Cont_PageSplitssec_FIN)


	-- Memory Grants Pending 
	-- número de consultas que están esperando asignación de memoria antes de poder ejecutarse. Ultimo valor observado
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)	
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Memory Grants Pending (consultas esperando asignación de memoria)' ,convert(varchar,cntr_value)
	FROM sys.dm_os_performance_counters 
	WHERE counter_name = 'Memory Grants Pending' AND object_name = @CounterPrefix +'Memory Manager'

	-- Free Memory (KB)                                                                                                                
	-- Cantidad de memoria libre disponible en el sistema. Ultimo valor observado
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)	
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Free Memory (KB) (memoria libre disponible)' ,convert(varchar,cntr_value)
	FROM sys.dm_os_performance_counters 
	WHERE counter_name = 'Free Memory (KB)' AND object_name = @CounterPrefix +'Memory Manager'

	-- Optimizer Memory (KB)                                                                                                           
	-- Cantidad de memoria que el Query Optimizer está utilizando para procesar y optimizar consultas. Ultimo valor observado. ¿consultas complejas?
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)	
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Optimizer Memory (KB) (memoria utilizada por el Optimizador de Consultas)' ,convert(varchar,cntr_value)
	FROM sys.dm_os_performance_counters 
	WHERE counter_name = 'Optimizer Memory (KB)' AND object_name = @CounterPrefix +'Memory Manager'

	-- User Connections
	-- cantidad de conexiones de usuario activas en el servidor.  Ultimo valor observado
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)	
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'User Connections (cantidad de conexiones de usuario activas en el servidor)' ,convert(varchar,cntr_value)
	FROM sys.dm_os_performance_counters 
	WHERE counter_name = 'User Connections' AND object_name = @CounterPrefix +'General Statistics'

	-- Transactions                                                                                                                    
	-- cantidad de transacciones activas en el servidor.  Ultimo valor observado
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)	
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Transactions (cantidad de transacciones activas en el servidor)' ,convert(varchar,cntr_value)
	FROM sys.dm_os_performance_counters 
	WHERE counter_name = 'Transactions' AND object_name = @CounterPrefix +'General Statistics'

	-- Processes blocked
	-- cantidad de procesos que están bloqueados en un momento determinado
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)	
	SELECT 'Comprobaciones del Servidor','Instancia de SQL Server','Contadores de Rendimiento', 'Processes blocked (cantidad de procesos bloqueados en un momento dado)' ,convert(varchar,cntr_value)
	FROM sys.dm_os_performance_counters 
	WHERE counter_name = 'Processes blocked' AND object_name = @CounterPrefix +'General Statistics'

	-- esperas acumulativas
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)	
	SELECT TOP (@TopN) 'Comprobaciones del Servidor','Instancia de SQL Server','Esperas Acumulativas', 'Top ' + convert(varchar,@TopN) +' de Esperas ordenadas por tiempo de espera ', convert(varchar,wait_type ) + ' (' + convert(varchar,convert(decimal(18,2),100.*wait_time_ms/SUM(wait_time_ms) OVER ()))+ '%)'
	FROM sys.dm_os_wait_stats              
	WHERE wait_type NOT IN ( 'SP_SERVER_DIAGNOSTICS_SLEEP','SLEEP_TASK', 'BROKER_TASK_STOP', 'BROKER_TO_FLUSH',   'SQLTRACE_BUFFER_FLUSH','CLR_AUTO_EVENT', 'CLR_MANUAL_EVENT',    'LAZYWRITER_SLEEP', 'SLEEP_SYSTEMTASK', 'SLEEP_BPOOL_FLUSH',   'BROKER_EVENTHANDLER', 'XE_DISPATCHER_WAIT', 'FT_IFTSHC_MUTEX',   'CHECKPOINT_QUEUE', 'FT_IFTS_SCHEDULER_IDLE_WAIT',    'BROKER_TRANSMITTER', 'FT_IFTSHC_MUTEX', 'KSOURCE_WAKEUP',   'LOGMGR_QUEUE', 'ONDEMAND_TASK_QUEUE',   'REQUEST_FOR_DEADLOCK_SEARCH', 'XE_TIMER_EVENT', 'BAD_PAGE_PROCESS',   'DBMIRROR_EVENTS_QUEUE', 'BROKER_RECEIVE_WAITFOR',   'PREEMPTIVE_OS_GETPROCADDRESS', 'PREEMPTIVE_OS_AUTHENTICATIONOPS',   'WAITFOR', 'DISPATCHER_QUEUE_SEMAPHORE', 'XE_DISPATCHER_JOIN',   'RESOURCE_QUEUE' ) 
		AND wait_time_ms > 0 
	ORDER BY wait_time_ms DESC





	-- autocrecimiento de las bases de datos
	DECLARE @filename NVARCHAR(1000);
	DECLARE @bc INT;
	DECLARE @ec INT;
	DECLARE @bfn VARCHAR(1000);
	DECLARE @efn VARCHAR(10);
	SELECT @filename = CAST(value AS NVARCHAR(1000)) FROM ::fn_trace_getinfo(DEFAULT) WHERE traceid = 1 AND property = 2;
	SET @filename = REVERSE(@filename);
	SET @bc = CHARINDEX('.',@filename);
	SET @ec = CHARINDEX('_',@filename)+1;
	SET @efn = REVERSE(SUBSTRING(@filename,1,@bc));
	SET @bfn = REVERSE(SUBSTRING(@filename,@ec,LEN(@filename)));
	SET @filename = @bfn + @efn

	SELECT ftg.StartTime,te.name AS EventName,DB_NAME(ftg.databaseid) AS DatabaseName,ftg.Filename,(ftg.IntegerData*8)/1024.0 AS GrowthMB,(ftg.duration/1000)AS DurMS
	INTO #CrecimientosBD
	FROM ::fn_trace_gettable(@filename, DEFAULT) AS ftg INNER JOIN sys.trace_events AS te ON ftg.EventClass = te.trace_event_id  
	WHERE (ftg.EventClass = 92  -- Date File Auto-grow
    	OR ftg.EventClass = 93) -- Log File Auto-grow
	ORDER BY ftg.StartTime


	-- comprobar propiedades de la base de datos de sistema Tempdb----------------------------------------------------------------------------------------------------------------------------------
	--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,valor)
	SELECT  'Comprobaciones de base de datos','TempDB','Ficheros de datos y transacciones', 'Nº de ficheros de datos',COUNT(file_id) 
	FROM tempdb.sys.database_files WHERE Type=0

	-- tamaño total
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	EXEC('USE TempDB;SELECT ''Comprobaciones de base de datos'',''TempDB'',''Ficheros de datos y transacciones'',''Tamaño Total de los ficheros de datos'', convert(varchar,convert(decimal(16,2),(SUM(size)*8)/1024.0)) + '' MB con un espacio en uso de '' +convert(varchar,convert(decimal(16,2),(SUM(FILEPROPERTY([name],''SpaceUsed''))*8)/1024.0)) +'' MB''
	FROM tempdb.sys.database_files
	WHERE type=0') 
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	EXEC('USE TempDB;SELECT ''Comprobaciones de base de datos'',''TempDB'',''Ficheros de datos y transacciones'',''Tamaño Total de los ficheros de transacciones'', convert(varchar,convert(decimal(16,2),(SUM(size)*8)/1024.0)) + '' MB con un espacio en uso de '' +convert(varchar,convert(decimal(16,2),(SUM(FILEPROPERTY([name],''SpaceUsed''))*8)/1024.0)) +'' MB''
	FROM tempdb.sys.database_files
	WHERE type<>0')

	-- tamaños y crecimientos desiguales
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT  'Comprobaciones de base de datos','TempDB','Ficheros de datos y transacciones', 'Tamaños distintos en los ficheros de datos', CASE WHEN count(distinct size) =1 THEN 'Solo existe un tamaño definido en los ficheros' ELSE 'Existen ' + convert(varchar,count(distinct size)) + ' tamaños diferentes definidos' END
	FROM tempdb.sys.database_files
	WHERE Type=0 
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT  'Comprobaciones de base de datos','TempDB','Ficheros de datos y transacciones', 'Crecimientos distintos en los ficheros de datos',CASE WHEN count(distinct growth) =1 THEN 'Solo existe un crecimiento definido en los ficheros' ELSE  'Existen ' +convert(varchar,count(distinct growth))+ ' crecimientos diferentes definidos' END
	FROM tempdb.sys.database_files
	WHERE Type=0 

	-- crecimientos en porcentajes
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT  'Comprobaciones de base de datos','TempDB','Ficheros de datos y transacciones', 'Crecimientos definidos en porcentajes','Existen ' +convert(varchar,count(name))+ ' ficheros con crecimientos definidos en porcentajes' 
	FROM tempdb.sys.database_files
	WHERE is_percent_growth=1

	
	-- Localizaciónes, que esten en C o en red
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones de base de datos','TempDB','Localizaciones','Localización de ficheros de datos (*.mdf)',physical_name
	FROM tempdb.sys.database_files 
	WHERE type_desc = 'ROWS' --AND ((UPPER(LEFT(physical_name, 1)) = 'C' OR physical_name LIKE '\\%'))
	UNION
	SELECT 'Comprobaciones de base de datos','TempDB','Localizaciones','Localización de ficheros de transacciones (*.ldf)',physical_name
	FROM tempdb.sys.database_files 
	WHERE type_desc <> 'ROWS'

	-- autocrecimientos
	IF NOT EXISTS (SELECT 1 FROM #CrecimientosBD WHERE DatabaseName ='tempdb')
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		SELECT 'Comprobaciones de base de datos','TempDB','Auto Crecimientos','Auto Crecimientos registrados en traza defecto','0'
	
	ELSE BEGIN			
		SELECT @Detalles=''
		SELECT @Detalles=@Detalles + 'El fichero de transacciones '+ Filename 
			+ ' registro ' + CASE WHEN count(1)=1 THEN convert(varchar,count(1)) + ' crecimiento de ' ELSE convert(varchar,count(1)) + ' crecimientos de ' END  + convert(varchar,convert(decimal(16,2),MAX(GrowthMB)))+ ' MB durante un máximo de '  + convert(varchar,MAX(DurMS)) + CASE WHEN  convert(varchar,MIN(StartTime),113)=convert(varchar,MAX(StartTime),113) THEN ' milisegundos entre el día ' +convert(varchar,MIN(StartTime),113) ELSE ' milisegundos entre los días ' + convert(varchar,MIN(StartTime),113) + ' y ' + convert(varchar,MAX(StartTime),113) END+ '.'+CHAR(13)
		FROM #CrecimientosBD
		WHERE DatabaseName ='tempdb' AND EventName='Log File Auto Grow'
		GROUP BY Filename

		SELECT @Detalles=@Detalles + 'El fichero de datos ' + Filename 
			+ ' registro ' + CASE WHEN count(1)=1 THEN convert(varchar,count(1)) + ' crecimiento de ' ELSE convert(varchar,count(1)) + ' crecimientos de ' END  + convert(varchar,convert(decimal(16,2),MAX(GrowthMB)))+ ' MB durante un máximo de '  + convert(varchar,MAX(DurMS))  + CASE WHEN  convert(varchar,MIN(StartTime),113)=convert(varchar,MAX(StartTime),113) THEN ' milisegundos entre el día ' +convert(varchar,MIN(StartTime),113) ELSE ' milisegundos entre los días ' + convert(varchar,MIN(StartTime),113) + ' y ' + convert(varchar,MAX(StartTime),113) END+ '.'+CHAR(13)
		FROM #CrecimientosBD
		WHERE DatabaseName ='tempdb' AND EventName<>'Log File Auto Grow'
		GROUP BY Filename

		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		VALUES ('Comprobaciones de base de datos','TempDB','Auto Crecimientos','Auto Crecimientos registrados en traza defecto',@Detalles)
	END							

	-- contadores
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones de base de datos','TempDB','Contadores de Rendimiento', 'Log Flush Wait Time últimos ' + convert(varchar,@SegMedicionContadores)+ ' segs (milisegundos que se tarda en escribir el log a disco) ',convert(varchar,@Cont_LogFlushWaitTime_FIN_TempDB)
	
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones de base de datos','TempDB','Contadores de Rendimiento', 'Transactions/sec últimos ' + convert(varchar,@SegMedicionContadores)+ ' segs (cantidad de transacciones completadas por segundo) ',convert(varchar,@Cont_Transactionssec_FIN_TempDB)
	
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones de base de datos','TempDB','Contadores de Rendimiento', 'Log Flushes/sec últimos ' + convert(varchar,@SegMedicionContadores)+ ' segs (veces por segundo que el log se descarga a disco) ',convert(varchar,@Cont_LogFlushessec_FIN_TempDB)

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones de base de datos','TempDB','Contadores de Rendimiento', 'Temp Tables Creation Rate últimos ' + convert(varchar,@SegMedicionContadores)+ ' segs (nº de tablas temporales creadas por segundo) ',convert(varchar,@Cont_TempTablesCreationRate_Fin)


	-- Active Transactions. consultas complejas? ultimo valor observado
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones de base de datos','TempDB','Contadores de Rendimiento', 'Active Transactions (transacciones activas)' , convert(varchar,cntr_value)
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name =  'Active Transactions' AND object_name = @CounterPrefix +'Databases' AND instance_name='TempDB'


	-- comprobar propiedades de la base de datos---------------------------------------------------------------------------------------------------------------------------------------------
	------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
	-- tamaño
	SELECT @lSql='USE [' + @BDActual+']; SELECT ''Comprobaciones de base de datos'',''' +@BDActual+ ''',''Ficheros de datos y transacciones'',''Tamaño Total de los ficheros de datos'', convert(varchar,convert(decimal(16,2),(SUM(size)*8)/1024.0)) + '' MB con un espacio en uso de '' + convert(varchar,convert(decimal(16,2),(SUM(FILEPROPERTY([name],''SpaceUsed''))*8)/1024.0)) +'' MB''
	FROM sys.database_files
	WHERE [type]=0'
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	EXEC (@lSql)

	SELECT @lSql='USE [' + @BDActual+']; SELECT ''Comprobaciones de base de datos'',''' +@BDActual+ ''',''Ficheros de datos y transacciones'',''Tamaño Total de los ficheros de transacciones'', convert(varchar,convert(decimal(16,2),(SUM(size)*8)/1024.0)) + '' MB con un espacio en uso de '' + convert(varchar,convert(decimal(16,2),(SUM(FILEPROPERTY([name],''SpaceUsed''))*8)/1024.0)) +'' MB''
	FROM sys.database_files
	WHERE [type]<>0'
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	EXEC (@lSql)


	-- Localización de las bases de datos
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT  'Comprobaciones de base de datos',@BDActual,'Localizaciones','Localización de ficheros de datos (*.mdf)',physical_name
	FROM sys.master_files
	WHERE type_desc = 'ROWS' AND DB_NAME(database_id)= @BDActual --AND ((UPPER(LEFT(physical_name, 1)) = 'C' OR physical_name LIKE '\\%'))
	UNION
	SELECT 'Comprobaciones de base de datos',@BDActual,'Localizaciones','Localización de ficheros de transacciones (*.ldf)',physical_name
	FROM sys.master_files
	WHERE type_desc <> 'ROWS' AND DB_NAME(database_id)= @BDActual


	-- autocrecimientos 
	IF NOT EXISTS (SELECT 1 FROM #CrecimientosBD WHERE DatabaseName =@BDActual)
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		SELECT 'Comprobaciones de base de datos',@BDActual,'Auto Crecimientos','Auto Crecimientos registrados en traza defecto','0'
	
	ELSE BEGIN
		SELECT @Detalles=''
		SELECT @Detalles=@Detalles + 'El fichero de transacciones '+ Filename 
			+ ' registro ' + CASE WHEN count(1)=1 THEN convert(varchar,count(1)) + ' crecimiento de ' ELSE convert(varchar,count(1)) + ' crecimientos de ' END + convert(varchar,convert(decimal(16,2),MAX(GrowthMB)))+ ' MB durante un máximo de '  + convert(varchar,MAX(DurMS)) + CASE WHEN  convert(varchar,MIN(StartTime),113)=convert(varchar,MAX(StartTime),113) THEN ' milisegundos entre el día ' +convert(varchar,MIN(StartTime),113) ELSE ' milisegundos entre los días ' + convert(varchar,MIN(StartTime),113) + ' y ' + convert(varchar,MAX(StartTime),113) END + '.' +CHAR(13)
		FROM #CrecimientosBD
		WHERE DatabaseName =@BDActual AND EventName='Log File Auto Grow'
		GROUP BY Filename

		SELECT @Detalles=@Detalles + 'El fichero de datos ' + Filename 
			+ ' registro ' + CASE WHEN count(1)=1 THEN convert(varchar,count(1)) + ' crecimiento de ' ELSE convert(varchar,count(1)) + ' crecimientos de ' END + convert(varchar,convert(decimal(16,2),MAX(GrowthMB)))+ ' MB durante un máximo de '  + convert(varchar,MAX(DurMS))  + CASE WHEN  convert(varchar,MIN(StartTime),113)=convert(varchar,MAX(StartTime),113) THEN ' milisegundos entre el día ' +convert(varchar,MIN(StartTime),113) ELSE ' milisegundos entre los días ' + convert(varchar,MIN(StartTime),113) + ' y ' + convert(varchar,MAX(StartTime),113) END+ '.' +CHAR(13)
		FROM #CrecimientosBD
		WHERE DatabaseName =@BDActual AND EventName<>'Log File Auto Grow'
		GROUP BY Filename


		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		VALUES ('Comprobaciones de base de datos',@BDActual,'Auto Crecimientos','Auto Crecimientos registrados en traza defecto',@Detalles)
	END

	-- tamaños del log superiores a los datos
	-- solo sale si esta mal --
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones de base de datos',name,'Ficheros de datos y transacciones', 'Ficheros de transacciones (.ldf) de mayor tamaño que sus ficheros de datos (.mdf)',
			'Tamaño de los fichero de datos: ' + convert(varchar,(D.TamDatos*8)/1024) + ' MB. Tamaño de los ficheros de transacciones: ' +convert(varchar,(L.size*8)/1024)+ ' MB'
	FROM sys.master_files L
		CROSS APPLY (SELECT SUM(CAST(size AS BIGINT)) TamDatos FROM sys.master_files WHERE database_id = L.database_id AND type = 0) D
	WHERE DB_NAME(l.database_id) = @BDActual AND L.type=1 AND D.TamDatos < L.size 

	-- contadores
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones de base de datos',@BDActual,'Contadores de Rendimiento', 'Log Flush Wait Time últimos ' + convert(varchar,@SegMedicionContadores)+ ' segs (milisegundos que se tarda en escribir el log a disco) ',convert(varchar,@Cont_LogFlushWaitTime_FIN)

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones de base de datos',@BDActual,'Contadores de Rendimiento', 'Transactions/sec últimos ' + convert(varchar,@SegMedicionContadores)+ ' segs (cantidad de transacciones completadas por segundo) ',convert(varchar,@Cont_Transactionssec_FIN)
	
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones de base de datos',@BDActual,'Contadores de Rendimiento', 'Log Flushes/sec últimos ' + convert(varchar,@SegMedicionContadores)+ ' segs (veces por segundo que el log se descarga a disco) ',convert(varchar,@Cont_LogFlushessec_FIN)


	-- Active Transactions ultimo valor observado
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones de base de datos',@BDActual,'Contadores de Rendimiento', 'Active Transactions (transacciones activas)' , convert(varchar,cntr_value)
	FROM sys.dm_os_performance_counters 
	WHERE  counter_name =  'Active Transactions' AND object_name = @CounterPrefix +'Databases' AND instance_name=@BDActual


	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	-- collation_name
	SELECT 	'Comprobaciones de base de datos',name,'Propiedades','Intercalación diferente de la de TempDB','1'
	FROM sys.databases
	WHERE collation_name<>@collation_name_tempdb AND name = @BDActual
	UNION
	SELECT 	'Comprobaciones de base de datos',name,'Propiedades','Intercalación igual a la de TempDB','1'
	FROM sys.databases
	WHERE collation_name=@collation_name_tempdb AND name = @BDActual

	-- compatibility_level
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 	'Comprobaciones de base de datos',name,'Propiedades','Nivel de Compatibilidad', convert(varchar,compatibility_level)
	FROM sys.databases 
	WHERE name = @BDActual
	-- is_auto_close_on
	UNION
	SELECT 	'Comprobaciones de base de datos',name,'Propiedades', 'Cierre Automático activo', convert(varchar,is_auto_close_on)
	FROM sys.databases
	WHERE name = @BDActual
	-- is_auto_shrink_on activado
	UNION
	SELECT 'Comprobaciones de base de datos',name,'Propiedades', 'Reducción Automatica activa',convert(varchar,is_auto_shrink_on)
	FROM sys.databases
	WHERE name = @BDActual
	-- is_auto_create_stats_on
	UNION
	SELECT 'Comprobaciones de base de datos',name,'Propiedades', 'Creación Automática de Estadísticas activa',convert(varchar,is_auto_create_stats_on) 	
	FROM sys.databases
	WHERE name= @BDActual
	-- is_auto_update_stats_on 
	UNION
	SELECT 'Comprobaciones de base de datos',name,'Propiedades', 'Actualización Automática de Estadísticas activa',convert(varchar,is_auto_update_stats_on)
	FROM sys.databases
	WHERE name = @BDActual
	-- is_local_cursor_default
	UNION
	SELECT 'Comprobaciones de base de datos',name,'Propiedades', 'Cursor Local por Defecto activo',convert(varchar,is_local_cursor_default)
	FROM sys.databases
	WHERE name = @BDActual
	-- is_recursive_triggers_on
	UNION
	SELECT 'Comprobaciones de base de datos',name,'Propiedades', 'Triggers Recursivos Habilitados',convert(varchar,is_recursive_triggers_on)
	FROM sys.databases
	WHERE name = @BDActual
	-- page_verify_option
	UNION
	SELECT 'Comprobaciones de base de datos',name,'Propiedades', 'Opción de Verificación de Páginas',convert(varchar,page_verify_option_desc)
	FROM sys.databases
	WHERE name = @BDActual
	--is_ansi_null_default_on
	UNION
	SELECT 'Comprobaciones de base de datos',name,'Propiedades', 'Opción ANSI NULLS por DEFECTO activa',convert(varchar,is_ansi_null_default_on)		
	FROM sys.databases
	WHERE name = @BDActual
	--is_ansi_nulls_on
	UNION
	SELECT 'Comprobaciones de base de datos',name,'Propiedades', 'Opción ANSI NULLS activa',convert(varchar,is_ansi_nulls_on)	
	FROM sys.databases
	WHERE name = @BDActual
	--is_ansi_padding_on
	UNION
	SELECT 'Comprobaciones de base de datos',name,'Propiedades', 'Opción ANSI Padding activa',convert(varchar,is_ansi_padding_on)	
	FROM sys.databases
	WHERE name = @BDActual
	--is_ansi_warnings_on
	UNION
	SELECT 'Comprobaciones de base de datos',name,'Propiedades', 'Opción ANSI Warnings activa',convert(varchar,is_ansi_warnings_on)	
	FROM sys.databases
	WHERE name = @BDActual
	--is_arithabort_on
	UNION
	SELECT 'Comprobaciones de base de datos',name,'Propiedades', 'Opción Arithmetic Abort activa',convert(varchar,is_arithabort_on)
	FROM sys.databases
	WHERE name = @BDActual
	--is_concat_null_yields_null_on
	UNION
	SELECT 'Comprobaciones de base de datos',name,'Propiedades', 'Opción Concatenate Null Yields Null desactivada',convert(varchar,is_concat_null_yields_null_on)
	FROM sys.databases
	WHERE name = @BDActual
	--is_quoted_identifier_on
	UNION
	SELECT 'Comprobaciones de base de datos',name,'Propiedades', 'Opción Quoted Identifiers desactivada',convert(varchar,is_quoted_identifier_on)
	FROM sys.databases
	WHERE name = @BDActual
	--is_numeric_roundabort_on a off
	UNION
	SELECT 'Comprobaciones de base de datos',name,'Propiedades', 'Opción Numeric Round Abort activada',convert(varchar,is_numeric_roundabort_on)
	FROM sys.databases
	WHERE name = @BDActual
	--recovery_model
	UNION
	SELECT 'Comprobaciones de base de datos',name,'Propiedades', 'Modelo de Recuperación',convert(varchar,recovery_model_desc)
	FROM sys.databases
	WHERE name = @BDActual
	


	-- indices y estadisticas -----------------------------------------------------------------------------------------------------------------------------------------------------------------------
	-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
	-- ultimas actualizaciones estadisticas
	SELECT @lSql='USE [' + @BDActual+'];SELECT S.name,OBJECT_NAME(S.object_id),STATS_DATE(S.object_id,S.stats_id),p.rows
		FROM sys.stats S 
		INNER JOIN sys.objects O ON S.object_id=O.object_id
		INNER JOIN sys.partitions P ON S.object_id = P.object_id AND P.index_id IN (0, 1)
		WHERE O.Type IN (''U'')'
	INSERT INTO #Estadisticas (NombreEstadistica,Tabla,UltimaActualizacion,NumRegistros)
	EXEC (@lSql)
		
	IF EXISTS (SELECT 1 FROM #Estadisticas GROUP BY Tabla,NumRegistros HAVING (MAX(UltimaActualizacion) <dateadd(dd,-7,GETDATE()) AND NumRegistros>1000) OR (MAX(UltimaActualizacion) IS NULL AND  NumRegistros>1000))
	BEGIN	
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		SELECT 'Comprobaciones de base de datos',@BDActual,'Índices y Estadisticas', 'Nº de Estadísticas no actualizadas en la última semana', convert(varchar,count(1)) 
		FROM #Estadisticas 

	END ELSE BEGIN
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		SELECT 'Comprobaciones de base de datos',@BDActual,'Índices y Estadisticas', 'Nº de Estadísticas No actualizadas en la última semana','0'
	END

	-- fragmentacion indices 
	SELECT @lSql='USE [' + @BDActual+'];SELECT PS.AVG_FRAGMENTATION_IN_PERCENT PorcentFragmentacion,IND.NAME Indice,OBJECT_NAME(IND.OBJECT_ID) Tabla 
	FROM SYS.DM_DB_INDEX_PHYSICAL_STATS (DB_ID(''' + @BDActual + '''),NULL,NULL,NULL,NULL) PS
	INNER JOIN SYS.INDEXES IND ON PS.OBJECT_ID = IND.OBJECT_ID AND PS.INDEX_ID = IND.INDEX_ID
	WHERE PS.index_id > 0 /*0 monton*/ AND PS.AVG_FRAGMENTATION_IN_PERCENT > 30'

	INSERT INTO #Indices (PorcFragmentacion,Indice,Tabla)
	EXEC (@lSql)

	IF EXISTS (SELECT 1 FROM #Indices) BEGIN
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		SELECT 'Comprobaciones de base de datos',@BDActual,'Índices y Estadisticas', 'Nº Índices con Fragmentación superior al 30%', convert(varchar,count(1))
		FROM #Indices

	END ELSE BEGIN
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		SELECT 'Comprobaciones de base de datos',@BDActual,'Índices y Estadisticas', 'Nº Índices con Fragmentación superior al 30%','0'
	END

	--SET @Script='USE [' + @BDActual+ '];'+ CHAR(13) 
	--SET @Detalles='Compruebe la fecha de reinicio del servidor de SQL antes de borrar el índice, un reinicio reciente no arroja información suficiente.' + CHAR(13) 

	-- indices que no se usan y si se actualizan	
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones de base de datos',@BDActual,'Índices y Estadisticas', 'Índices que No se Usan y si se Actualizan', convert(varchar,count(1))
	FROM sys.dm_db_index_usage_stats AS ius JOIN sys.indexes AS i ON i.index_id = ius.index_id AND i.object_id = ius.object_id 
	WHERE ius.database_id = DB_ID(@BDActual) AND i.is_unique_constraint = 0 -- no unique indexes         
		AND i.is_primary_key = 0 AND i.is_disabled = 0 
		AND i.type > 1 -- don't consider heaps/clustered index   
		--AND ( ( ius.user_seeks + ius.user_scans + ius.user_lookups ) < ius.user_updates OR ( ius.user_seeks = 0 AND ius.user_scans = 0 ) )
		AND  ( ius.user_seeks + ius.user_scans + ius.user_lookups ) =0 AND ius.user_updates >0 

	-- indices faltantes 
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones de base de datos',@BDActual,'Índices y Estadisticas','Índices Faltantes',convert(varchar,count(1))
	FROM sys.dm_db_missing_index_groups mig INNER JOIN sys.dm_db_missing_index_group_stats migs ON migs.group_handle = mig.index_group_handle 
		INNER JOIN sys.dm_db_missing_index_details mid ON mig.index_handle = mid.index_handle 
	WHERE (mid.database_id)=DB_ID(@BDActual )
	

	-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------	
	-- sesiones ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
	-- usuarios licencia y usuarios Ahora_Sesion
	IF EXISTS (SELECT 1	FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME = 'Empleados_Datos_Licencias') BEGIN

		SET @Script=N'SELECT @ret=COUNT(1) FROM ['+ @BDActual +']..Empleados_Datos_Licencias '
		EXECUTE sp_executesql @Script,N'@ret int OUTPUT',@ret=@Cuenta output

		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		SELECT 'Comprobaciones de base de datos',@BDActual,'Usuarios ERP (licencias)', 'Nº de usuarios ERP' , convert(varchar,@Cuenta) 

	END	
	
	IF EXISTS (SELECT 1	FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME = 'Ahora_Sesion') BEGIN

		SET @Script=N'SELECT @ret=COUNT(1) FROM ['+ @BDActual +']..Ahora_Sesion '
		EXECUTE sp_executesql @Script,N'@ret int OUTPUT',@ret=@Cuenta output

		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		SELECT 'Comprobaciones de base de datos',@BDActual,'Usuarios Ahora_Sesion', 'Nº de usuarios Ahora_Sesion' , convert(varchar,@Cuenta) 
	END		
	
	-- nº de sesiones activas en la bd
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones de base de datos',@BDActual,'Sesiones', 'Nº de sesiones activas ' , convert(varchar,count(distinct s.session_id)) 
	FROM sys.dm_exec_sessions s JOIN sys.dm_exec_connections c ON s.session_id = c.session_id         
	CROSS APPLY sys.dm_exec_sql_text(c.most_recent_sql_handle) t 
	WHERE DB_name(database_id)=@BDActual AND s.session_id  <>@@SPID

	-- sesiones suspendidas en la bd
	-- solo sale si esta mal --
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones de base de datos',@BDActual,'Sesiones', 'Sesiones suspendidas ' , 
		'El usuario: '+ sess.login_name + ' con SPID: '   + convert(varchar,r.session_id) +  ' mantiene una sesion suspendida'  + CASE WHEN r.blocking_session_id>0 THEN '  por el SPID: ' + convert(varchar,r.blocking_session_id) ELSE '' END 
		+  CASE WHEN r.open_transaction_count >0 THEN '  con ' + convert(varchar, r.open_transaction_count)+ ' transacciones activas' ELSE '' END 
		+ '  desde el equipo ' + sess.host_name  + CASE WHEN wt.wait_type IS NOT NULL THEN wt.wait_type ELSE '' END
		--,'Texto: ' + S.text 	
	FROM sys.dm_exec_requests r CROSS APPLY sys.dm_exec_sql_text(r.sql_handle) s
		left JOIN sys.dm_exec_sessions sess ON r.session_id = sess.session_id
		left join sys.dm_os_waiting_tasks AS wt ON wt.session_id = sess.session_id
	WHERE db_name(r.database_id)=@BDActual AND r.session_id  <>@@SPID AND r.status = N'suspended';  

	-- transacciones activas por mas de 30 segundos
	-- solo sale si esta mal --
	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT 'Comprobaciones de base de datos',@BDActual,'Sesiones', 'Transacciones activas por mas de 30 segundos','El usuario: '+ sess.login_name + ' con SPID: ' + convert(varchar,st.session_id) + '  mantiene ' + convert(varchar,st.open_transaction_count) + CASE WHEN st.open_transaction_count >1 THEN ' transacciones activas de ' ELSE ' transacción activa de ' END + convert(varchar,DATEDIFF(SECOND, at.transaction_begin_time, GETDATE())) + ' segundos desde el equipo ' + host_name 
	FROM sys.dm_tran_active_transactions at INNER JOIN sys.dm_tran_session_transactions st ON st.transaction_id = at.transaction_id
	  LEFT OUTER JOIN sys.dm_exec_sessions sess ON st.session_id = sess.session_id
	  LEFT OUTER JOIN sys.dm_exec_connections conn ON conn.session_id = sess.session_id
		OUTER APPLY sys.dm_exec_sql_text(conn.most_recent_sql_handle)  AS txt
	WHERE db_name(sess.database_id)=@BDActual AND st.session_id  <>@@SPID
	GROUP BY st.session_id,st.open_transaction_count,login_name,host_name,PROGRAM_NAME,status,text,at.transaction_begin_time
	HAVING DATEDIFF(SECOND, at.transaction_begin_time, GETDATE()) >30  --mas de 30 segundos
	ORDER BY  DATEDIFF(SECOND, at.transaction_begin_time, GETDATE()) DESC;

	
	-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------	
	-- planes cacheados ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
	SELECT OBJECT_NAME (st.objectid) Objeto,db_name(st.dbid) Tipo, cp.refcounts Num_Referencias,usecounts Num_Busquedas,convert(decimal(38,2),size_in_bytes/1000.0) Tamaño_KB,
		objtype Tipo_Objeto,creation_time Fecha_CreacionPlan,deqs.last_execution_time Fecha_UltimaEjecucionPlan, deqs.execution_count Num_Ejecuciones,deqs.total_rows Num_Filas, 
		deqs.total_grant_kb
		,deqs.total_elapsed_time/1000 TiempoTotal_ms
		,deqs.total_logical_reads Num_Total_LecturasLogicas
		,deqs.total_logical_writes Num_Total_EscriturasLogicas
		,deqs.total_worker_time/1000 TiempoTotal_CPU_ms
		,st.text Texto_Objeto,SUBSTRING(st.text, (deqs.statement_start_offset/2) + 1, ((CASE deqs.statement_end_offset WHEN -1 THEN DATALENGTH(st.text) ELSE deqs.statement_end_offset END - deqs.statement_start_offset)/2) + 1) AS Sentencia_TextoObjeto
		--,p.query_plan Plan_Cacheado
	INTO #PlanesCachados
	FROM sys.dm_exec_cached_plans AS cp 
		INNER JOIN sys.dm_exec_query_stats AS deqs ON cp.plan_handle = deqs.plan_handle --OUTER APPLY sys.dm_exec_query_plan(cp.plan_handle) p 
		OUTER APPLY sys.dm_exec_sql_text(cp.plan_handle) AS st 
	WHERE db_name(st.dbid)=@BDActual --OR db_name(st.dbid)='TempDB'
		AND (OBJECT_NAME (st.objectid) <>'ZZZ_Ahora_System_Health' /*OR st.objectid IS NULL*/)


	-- vigencia de los planes
	SELECT TOP 1 @FechaMinPlanCache= Fecha_CreacionPlan FROM #PlanesCachados ORDER BY Fecha_CreacionPlan
	IF @FechaMinPlanCache IS NULL 
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		SELECT 'Comprobaciones Cache de Planes de Ejecución',@BDActual,'Datos generales Cache','Vigencia de los planes','No Existen planes de ejecución cacheados para la base de datos '		
	ELSE BEGIN

		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		SELECT TOP 1 'Comprobaciones Cache de Planes de Ejecución',@BDActual,'Datos generales Cache','Vigencia de los planes','Existen planes de ejecución desde el ' + convert(varchar,@FechaMinPlanCache,120)
	
		-- planes de un solo uso
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		SELECT 'Comprobaciones Cache de Planes de Ejecución',@BDActual,'Datos generales Cache','Planes de un solo uso o consultas Adhoc', convert(varchar,COUNT(1)) 
		FROM #PlanesCachados
		WHERE (Num_Ejecuciones=1 OR Tipo_Objeto='Adhoc') 
		HAVING COUNT(1)>0
		UNION 
		SELECT 'Comprobaciones Cache de Planes de Ejecución',@BDActual,'Datos generales Cache','Planes de un solo uso o consultas Adhoc','0'
		FROM #PlanesCachados
		WHERE (Num_Ejecuciones=1 OR Tipo_Objeto='Adhoc') 
		HAVING COUNT(1)=0

		--TOP 5 ordenado por CPU
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		SELECT TOP (@TopN) 'Comprobaciones Cache de Planes de Ejecución',@BDActual,'Top '+ convert(varchar,@TopN) +' Consultas ordenadas por CPU',
			CASE WHEN Objeto IS NOT NULL THEN 'Objeto: ' + Objeto + '. ' ELSE '' END  + 'Tiempo total '+ convert(varchar,TiempoTotal_ms) + ' ms. Tiempo total CPU '+ convert(varchar,TiempoTotal_CPU_ms) + ' ms. Nº Lecturas ' + convert(varchar,Num_Total_LecturasLogicas), Sentencia_TextoObjeto
		FROM #PlanesCachados
		ORDER BY [TiempoTotal_CPU_ms] DESC

		--TOP 5 ordenado por lecturas
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		SELECT TOP (@TopN) 'Comprobaciones Cache de Planes de Ejecución',@BDActual,'Top '+ convert(varchar,@TopN) +' Consultas ordenadas por Lecturas (paginas)',
			CASE WHEN Objeto IS NOT NULL THEN 'Objeto: ' + Objeto + '. ' ELSE '' END  + 'Tiempo total '+ convert(varchar,TiempoTotal_ms) + ' ms. Tiempo total CPU '+ convert(varchar,TiempoTotal_CPU_ms) + ' ms. Nº Lecturas ' + convert(varchar,Num_Total_LecturasLogicas) ,  Sentencia_TextoObjeto 
		FROM #PlanesCachados
		ORDER BY [Num_Total_LecturasLogicas] DESC

		--TOP 5 ordenado por tiempo total
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		SELECT TOP (@TopN) 'Comprobaciones Cache de Planes de Ejecución',@BDActual,'Top '+ convert(varchar,@TopN) +' Consultas ordenadas por Tiempo',
			CASE WHEN Objeto IS NOT NULL THEN 'Objeto: ' + Objeto + '. ' ELSE '' END  + 'Tiempo total '+ convert(varchar,TiempoTotal_ms) + ' ms. Tiempo total CPU '+ convert(varchar,TiempoTotal_CPU_ms) + ' ms. Nº Lecturas ' + convert(varchar,Num_Total_LecturasLogicas) ,  Sentencia_TextoObjeto
		FROM #PlanesCachados
		ORDER BY [TiempoTotal_ms] DESC

		--TOP 5 ordenado por Num_ de ejecuciones
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		SELECT TOP (@TopN) 'Comprobaciones Cache de Planes de Ejecución',@BDActual,'Top '+ convert(varchar,@TopN) +' Consultas ordenadas Nº de Ejecuciones',
			CASE WHEN Objeto IS NOT NULL THEN 'Objeto: ' + Objeto + '. ' ELSE '' END  + 'Tiempo total '+ convert(varchar,TiempoTotal_ms) + ' ms. Tiempo total CPU '+ convert(varchar,TiempoTotal_CPU_ms) + ' ms. Nº Lecturas ' + convert(varchar,Num_Total_LecturasLogicas) + '. Nº Ejecuciones ' + convert(varchar,Num_Ejecuciones) ,  Sentencia_TextoObjeto 
		FROM #PlanesCachados
		ORDER BY [Num_Ejecuciones] DESC

	END
		
	-- top 5 indices faltantes 
	--SELECT TOP (@TopN) @Detalles=@Detalles + 'Posible índice faltante sobre la tabla: '+ REPLACE (mid.statement,'['+ @BDActual + '].','') + CHAR(13)
	--	,@Script= @Script +	'CREATE INDEX [missing_index_' + CONVERT (VARCHAR, mig.index_group_handle) + '_' + CONVERT (VARCHAR, mid.index_handle) + '_' + LEFT(PARSENAME(mid.statement, 1), 32) + ']' + ' ON ' + mid.statement + ' (' + ISNULL(mid.equality_columns, '') + CASE WHEN mid.equality_columns IS NOT NULL AND mid.inequality_columns IS NOT NULL THEN ',' ELSE '' END + ISNULL(mid.inequality_columns, '') + ')' + ISNULL(' INCLUDE (' + mid.included_columns + ')', '') + ';'+ CHAR(13)
	--FROM sys.dm_db_missing_index_groups mig INNER JOIN sys.dm_db_missing_index_group_stats migs ON migs.group_handle = mig.index_group_handle 
	--	INNER JOIN sys.dm_db_missing_index_details mid ON mig.index_handle = mid.index_handle 
	--WHERE (mid.database_id)=DB_ID(@BDActual )
	--ORDER BY migs.avg_total_user_cost * ( migs.avg_user_impact / 100.0 ) * ( migs.user_seeks + migs.user_scans )  DESC

	INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
	SELECT TOP (@TopN) 'Comprobaciones Cache de Planes de Ejecución',@BDActual,'Top '+ convert(varchar,@TopN) +' Indices Faltantes ', 'Porcentaje de mejora ' +convert(varchar,CONVERT (decimal (28, 1),migs.avg_total_user_cost * (migs.avg_user_impact/ 100.0) * (migs.user_seeks + migs.user_scans)))			
		,'Tabla: '+ mid.statement + '     Clave del Índice: ' + ISNULL(mid.equality_columns, '') + CASE WHEN mid.inequality_columns IS NOT NULL THEN CASE WHEN mid.equality_columns IS NOT NULL THEN ', ' ELSE '' END + mid.inequality_columns ELSE '' END 
	FROM sys.dm_db_missing_index_groups mig INNER JOIN sys.dm_db_missing_index_group_stats migs ON migs.group_handle = mig.index_group_handle 
		INNER JOIN sys.dm_db_missing_index_details mid ON mig.index_handle = mid.index_handle 
	WHERE (mid.database_id)=DB_ID(@BDActual)
	ORDER BY migs.avg_total_user_cost * ( migs.avg_user_impact / 100.0 ) * ( migs.user_seeks + migs.user_scans )  DESC
	

	-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------	
	-- personalizaciones BD ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
	IF EXISTS (SELECT 1 FROM Sys.objects WHERE name='Actualizador_Objetos') BEGIN
		-- tablas 
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		EXEC('SELECT ''Comprobaciones personalización'', ''Personalizaciones en Base de Datos'',''Tablas'',''Nº de Tablas personalizadas'',convert(varchar,count(1))
				FROM ['+ @BDActual +'].sys.objects O LEFT JOIN ['+ @BDActual +']..Actualizador_Objetos A ON O.name=A.Name
				WHERE O.type=''U'' AND A.Id IS NULL')

		-- triggers
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		EXEC('SELECT ''Comprobaciones personalización'',''Personalizaciones en Base de Datos'',''Triggers'',''Nº de Triggers personalizados'',convert(varchar,count(1))
				FROM ['+ @BDActual +'].sys.objects O LEFT JOIN ['+ @BDActual +']..Actualizador_Objetos A ON O.name=A.Name
				WHERE O.type=''TR'' AND A.Id IS NULL')

		-- vistas
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		EXEC('SELECT ''Comprobaciones personalización'',''Personalizaciones en Base de Datos'',''Vistas'',''Nº de Vistas personalizadas'',convert(varchar,count(1))
				FROM ['+ @BDActual +'].sys.objects O LEFT JOIN ['+ @BDActual +']..Actualizador_Objetos A ON O.name=A.Name
				WHERE O.type=''V'' AND A.Id IS NULL')

		-- procedimientos
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		EXEC('SELECT ''Comprobaciones personalización'',''Personalizaciones en Base de Datos'',''Procedimientos'',''Nº de Procedimientos personalizados'',convert(varchar,count(1))
				FROM ['+ @BDActual +'].sys.objects O LEFT JOIN ['+ @BDActual +']..Actualizador_Objetos A ON O.name=A.Name
				WHERE O.type=''P'' AND A.Id IS NULL')

		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		EXEC('SELECT ''Comprobaciones personalización'',''Personalizaciones en Base de Datos'',''Procs. Personalizables'',''Nº de Procedimientos Personalizables modificados'',convert(varchar,count(1))
				FROM ['+ @BDActual +'].sys.objects O INNER JOIN ['+ @BDActual +']..Actualizador_Objetos A ON O.name=A.Name
					INNER JOIN ['+ @BDActual +'].sys.sql_modules M ON M.object_id=O.object_id
				WHERE O.type=''P'' AND A.xtype=''P'' AND A.Noactualizar=1 
					AND (
						(charindex(''AS''+CHAR(13),A.Script)<>0 AND ABS(LEN(SUBSTRING(A.Script,charindex(''AS''+CHAR(13),A.Script),LEN(A.Script))) - LEN(SUBSTRING(M.definition,charindex(''AS''+CHAR(13),M.definition),LEN(M.definition)))) >=50)
						OR
						(charindex(''AS''+CHAR(13),A.Script)=0 AND charindex(''AS '',A.Script)<>0 AND ABS(LEN(SUBSTRING(A.Script,charindex(''AS '',A.Script),LEN(A.Script))) - LEN(SUBSTRING(M.definition,charindex(''AS '',M.definition),LEN(M.definition)))) >=50)
						)
		')
		-- funciones
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		EXEC('SELECT ''Comprobaciones personalización'', ''Personalizaciones en Base de Datos'',''Funciones'',''Nº de Funciones personalizadas'',convert(varchar,count(1))
				FROM ['+ @BDActual +'].sys.objects O LEFT JOIN ['+ @BDActual +']..Actualizador_Objetos A ON O.name=A.Name
				WHERE O.type IN (''FN'',''TF'',''IF'') AND A.Id IS NULL')

		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		EXEC('SELECT ''Comprobaciones personalización'',''Personalizaciones en Base de Datos'',''Funcs. Personalizables'',''Nº de Funciones Personalizables modificadas'',convert(varchar,count(1))
				FROM ['+ @BDActual +'].sys.objects O INNER JOIN ['+ @BDActual +']..Actualizador_Objetos A ON O.name=A.Name
					INNER JOIN ['+ @BDActual +'].sys.sql_modules M ON M.object_id=O.object_id
				WHERE O.type IN (''FN'',''TF'',''IF'')  AND A.xtype IN (''FN'',''TF'',''IF'') AND A.Noactualizar=1 
					AND (
						(charindex(''AS''+CHAR(13),A.Script)<>0 AND ABS(LEN(SUBSTRING(A.Script,charindex(''AS''+CHAR(13),A.Script),LEN(A.Script))) - LEN(SUBSTRING(M.definition,charindex(''AS''+CHAR(13),M.definition),LEN(M.definition)))) >=50)
						OR
						(charindex(''AS''+CHAR(13),A.Script)=0 AND charindex(''AS '',A.Script)<>0 AND ABS(LEN(SUBSTRING(A.Script,charindex(''AS '',A.Script),LEN(A.Script))) - LEN(SUBSTRING(M.definition,charindex(''AS '',M.definition),LEN(M.definition)))) >=50)
						)
		')
		
	END


	-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------		
	-- Estadísticas de código
	IF EXISTS (
		SELECT 1 
		FROM INFORMATION_SCHEMA.TABLES 
		WHERE TABLE_NAME = 'Ahora_Grids' 
	) BEGIN
		SET @Script=N'SELECT @ret=COUNT(1) FROM ['+ @BDActual +']..Ahora_Grids'
		EXECUTE sp_executesql @Script,N'@ret int OUTPUT',@ret=@Cuenta output
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		VALUES ('Comprobaciones personalización','Personalizaciones en Código', 'Grids', 'Nº de Grids personalizadas' , CONVERT(VARCHAR(50), @Cuenta))
		
		SET @Script=N'SELECT @ret=COUNT(1) FROM ['+ @BDActual +']..Ahora_Grids_Campos'
		EXECUTE sp_executesql @Script,N'@ret int OUTPUT',@ret=@Cuenta output
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		VALUES ('Comprobaciones personalización','Personalizaciones en Código', 'Campos en Grids', 'Nº de Campos en Grids personalizadas' , CONVERT(VARCHAR(50), @Cuenta))

		SET @Script=N'SELECT @ret=COUNT(1) FROM ['+ @BDActual +']..Ahora_Grids_Campos WHERE isnull(ReglaColor,'''') <> '''''
		EXECUTE sp_executesql @Script,N'@ret int OUTPUT',@ret=@Cuenta output
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		VALUES ('Comprobaciones personalización','Personalizaciones en Código', 'Reglas Color en Grids', 'Nº de Reglas Color en Grids' , CONVERT(VARCHAR(50), @Cuenta))
			
		SET @Script=N'SELECT @ret=COUNT(1) FROM ['+ @BDActual +']..Ahora_Controles_Personalizados WHERE Libreria <>''AhoraDDA'''
		EXECUTE sp_executesql @Script,N'@ret int OUTPUT',@ret=@Cuenta output
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		VALUES ('Comprobaciones personalización','Personalizaciones en Código', 'Controles', 'Nº de Controles personalizados' , CONVERT(VARCHAR(50), @Cuenta))
	END

	-- Pantallas_Configuracion_Scripts ******************************************************************************************************************
	IF EXISTS (
		SELECT 1 
		FROM INFORMATION_SCHEMA.TABLES 
		WHERE TABLE_NAME = 'Pantallas_Configuracion_Scripts' 
	)
	BEGIN
		SET @Script=N'SELECT @ret=COUNT(1) FROM ['+ @BDActual +']..Pantallas_Configuracion_Scripts '
		EXECUTE sp_executesql @Script,N'@ret int OUTPUT',@ret=@Cuenta output

		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		VALUES (
			'Comprobaciones personalización',  'Personalizaciones en Código',
			'Formularios', 
			'Nº Formularios personalizados ' ,convert(varchar,@Cuenta)
		)
	END


	-- Scripts ******************************************************************************************************************
	IF EXISTS (
		SELECT 1 
		FROM INFORMATION_SCHEMA.TABLES 
		WHERE TABLE_NAME = 'Scripts' 
	)
	BEGIN
		SET @Script=N'SELECT @ret=COUNT(1) FROM ['+ @BDActual +']..Scripts '
		EXECUTE sp_executesql @Script,N'@ret int OUTPUT',@ret=@Cuenta output

		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		VALUES ('Comprobaciones personalización', 'Personalizaciones en Código', 'Scripts', 'Nº Scripts personalizados' , CONVERT(VARCHAR(50), @Cuenta))

		SET @Script=N'SELECT @ret=COUNT(1)
				FROM (
					SELECT s.IdScript
					FROM ['+ @BDActual +']..Scripts s
					inner join ['+ @BDActual +']..Ahora_Links l on s.IdDoc = l.IdDocObjeto and l.Coleccion = ''Scripts''
					inner join ['+ @BDActual +']..Ahora_Nodos n on l.IdLink = n.IdLink
					inner join ['+ @BDActual +']..Ahora_Seguridad_Entidades_Asignacion seg on n.IdDoc = seg.IdDocEntidad and IdEntidad = ''Ahora_Nodo''
					inner join ['+ @BDActual +']..Ahora_Grupos_Asignacion asi on seg.IdGrupo = asi.IdGrupo
					inner join ['+ @BDActual +']..Empleados_Datos_Licencias edl on asi.IdEmpleado = edl.IdEmpleado
					GROUP BY s.IdScript
				) a'
		EXECUTE sp_executesql @Script,N'@ret int OUTPUT',@ret=@Cuenta output

		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		VALUES ('Comprobaciones personalización', 'Personalizaciones en Código', 'Scrts. en Uso', 'Nº Scripts personalizados en uso' , CONVERT(VARCHAR(50), @Cuenta))
	END

	-- DDA_Pantallas ******************************************************************************************************************
	IF EXISTS (
		SELECT 1 
		FROM INFORMATION_SCHEMA.TABLES 
		WHERE TABLE_NAME = 'DDA_Pantallas' 
	)
	BEGIN
		SET @Script=N'SELECT @ret=COUNT(1) FROM ['+ @BDActual +']..DDA_Pantallas '
		EXECUTE sp_executesql @Script,N'@ret int OUTPUT',@ret=@Cuenta output
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		VALUES ('Comprobaciones personalización', 'Personalizaciones en Código', 'DDAs', 'Nº de DDAs' , CONVERT(VARCHAR(50), @Cuenta))
	END

	-- DDA_Pantallas_Controles ******************************************************************************************************************
	IF EXISTS (
		SELECT 1 
		FROM INFORMATION_SCHEMA.TABLES 
		WHERE TABLE_NAME = 'DDA_Pantallas_Controles' 
	)
	BEGIN
		SET @Script=N'SELECT @ret=COUNT(1) FROM ['+ @BDActual +']..DDA_Pantallas_Controles '
		EXECUTE sp_executesql @Script,N'@ret int OUTPUT',@ret=@Cuenta output
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		VALUES ('Comprobaciones personalización', 'Personalizaciones en Código', 'DDAs Contrls',  'Nº de ControlesDDAs' , CONVERT(VARCHAR(50), @Cuenta))
	END

	-- FORMULAS
	-- Esquemas_Elementos_Parametros ******************************************************************************************************************
	IF EXISTS (
		SELECT 1 
		FROM INFORMATION_SCHEMA.TABLES 
		WHERE TABLE_NAME = 'Esquemas_Elementos_Parametros' 
	)
	BEGIN
		SET @Script=N'SELECT @ret=COUNT(1) FROM ['+ @BDActual +']..Esquemas_Elementos_Parametros WHERE Formula IS NOT NULL '
		EXECUTE sp_executesql @Script,N'@ret int OUTPUT',@ret=@Cuenta output
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		VALUES ('Comprobaciones personalización', 'Personalizaciones en Código', 'Fórmulas Parametros Esquemas',  'Nº de Parámetros Esquemas en Fórmulas' , CONVERT(VARCHAR(50), @Cuenta))
	END

	-- DefinicionProd_Campos ******************************************************************************************************************
	IF EXISTS (
		SELECT 1 
		FROM INFORMATION_SCHEMA.TABLES 
		WHERE TABLE_NAME = 'DefinicionProd_Campos' 
	)
	BEGIN
		SET @Script=N'SELECT @ret=COUNT(1) FROM ['+ @BDActual +']..DefinicionProd_Campos WHERE Formula is not null and IdDefProd <> ''F1'' '
		EXECUTE sp_executesql @Script,N'@ret int OUTPUT',@ret=@Cuenta output
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		VALUES ('Comprobaciones personalización', 'Personalizaciones en Código', 'Fórmulas Definición Producto',  'Nº de Definición Producto en Fórmulas ' , CONVERT(VARCHAR(50), @Cuenta))
	END


	-- Prevision_Formulas ******************************************************************************************************************
	IF EXISTS (
		SELECT 1 
		FROM INFORMATION_SCHEMA.TABLES 
		WHERE TABLE_NAME = 'Prevision_Formulas' 
	)
	BEGIN
		SET @Script=N'SELECT @ret=COUNT(1) FROM ['+ @BDActual +']..Prevision_Formulas WHERE Formula is not null and IdFormula > 3'
		EXECUTE sp_executesql @Script,N'@ret int OUTPUT',@ret=@Cuenta output
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		VALUES ('Comprobaciones personalización', 'Personalizaciones en Código', 'Fórmulas Previsiones e Intereses',  'Nº de Previsiones e Intereses en Fórmulas ' , CONVERT(VARCHAR(50), @Cuenta))
	END

	-- Presentaciones_Unidades_Relacion ******************************************************************************************************************
	IF EXISTS (
		SELECT 1 
		FROM INFORMATION_SCHEMA.TABLES 
		WHERE TABLE_NAME = 'Presentaciones_Unidades_Relacion' 
	)
	BEGIN
		SET @Script=N'SELECT @ret=COUNT(1) FROM ['+ @BDActual +']..Presentaciones_Unidades_Relacion WHERE Formula IS NOT NULL'
		EXECUTE sp_executesql @Script,N'@ret int OUTPUT',@ret=@Cuenta output
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		VALUES ('Comprobaciones personalización', 'Personalizaciones en Código', 'Fórmulas Presentaciones',  'Nº de Presentaciones en Fórmulas ' , CONVERT(VARCHAR(50), @Cuenta))

	END

	-- EXTERNO
	IF EXISTS (
		SELECT 1 
		FROM INFORMATION_SCHEMA.TABLES 
		WHERE TABLE_NAME = 'Ahora_Procesos' 
	)
	BEGIN
		SET @Script=N'SELECT @ret=COUNT(DISTINCT Libreria) FROM ['+ @BDActual +']..Ahora_Procesos WHERE Libreria like ''%externo%'' AND Libreria <> ''AhoraExterno'''
		EXECUTE sp_executesql @Script,N'@ret int OUTPUT',@ret=@Cuenta output
		INSERT INTO ZZZ_TabResult (Categoria,SubCategoria,Tipo,Detalles,Valor)
		VALUES ('Comprobaciones personalización', 'Personalizaciones en Código', 'Externos',  'Nº de Externos ' , CONVERT(VARCHAR(50), @Cuenta))
	END
	

	select * from ZZZ_TabResult order by id
	--select * from ZZZ_TabResult where Tipo='contadores de rendimiento' order by id
	
	-- devolver el JSON_Result
	SELECT @JSON_Result =(
		SELECT Categoria, SubCategoria, Tipo, Detalles, Valor 
		FROM ZZZ_TabResult 
		--where Tipo='contadores de rendimiento' 
		ORDER BY Id
		FOR JSON PATH
	)

	SELECT @JSON_Result=REPLACE(@JSON_Result, '},{', '},' + CHAR(13) + CHAR(10) + '{') 
	SELECT @JSON_Result




	-- guardarlo en un archivo (no funciona bien, mete caracteres raros)
	--IF @Path_JSON_Result IS NOT NULL BEGIN
	--	EXEC sp_configure 'show advanced options', 1;
	--	RECONFIGURE;
	--	EXEC sp_configure 'xp_cmdshell', 1;
	--	RECONFIGURE;
		
	--	SET @filePath = @Path_JSON_Result + 'ZZZ_TabResult_' + REPLACE(REPLACE(REPLACE(convert(varchar,getdate(),120),' ',''),'-',''),':','_') + '.json';
	--	-- esto no genera bien el fichero
	--	--SET @cmd = 'xp_cmdshell ''bcp "SELECT CAST(Categoria AS NVARCHAR(MAX)) Categoria, CAST(Valor AS NVARCHAR(MAX)) Valor FROM ' +@BDActual+  '..ZZZ_TabResult FOR JSON AUTO" queryout ' + @filePath + ' -w -T -S ' + convert(varchar(100),@@SERVERNAME) + ' '''
			
	--	-- lo copio a una tabla auxiliar
	--	IF OBJECT_ID ('ZZZ_TabResult_AUX') IS NOT NULL DROP TABLE ZZZ_TabResult_AUX 
	--	SELECT @JSON_Result JSON_Result INTO ZZZ_TabResult_AUX
	--	SET @cmd = 'xp_cmdshell ''bcp ' +@BDActual+ '.dbo.ZZZ_TabResult_AUX out ' + @filePath + ' -w -T -S ' + convert(varchar(100),@@SERVERNAME) + ' '''
		

	--	BEGIN TRY
	--		EXEC (@cmd)
	--	END TRY
	--	BEGIN CATCH
	--		SELECT 'Error al intentar generar el fichero JSON'
	--		SELECT @@ERROR
	--	END CATCH
	--END

	RETURN -1 
END TRY     

BEGIN CATCH
	IF @@TRANCOUNT >0 BEGIN
		ROLLBACK TRAN 
	END

	DECLARE @CatchError NVARCHAR(MAX)
	SET @CatchError=ERROR_MESSAGE()
	RAISERROR(@CatchError,12,1)

	RETURN 0

END CATCH